module ColorNest(colorNest) where

import SMLtree
import List
import Array

import IOExts

colorNest :: [SMLtree] -> [SMLtree]
colorNest = map nestChildren . concatMap optNesting . groupBy isColor'

nestChildren (TagNode tag children) = TagNode tag (colorNest children)
nestChildren leaf = leaf

isColor' a b = isColor a == isColor b

isColor (TagNode (Color _) _) = True
isColor _ = False

optNesting :: [SMLtree] -> [SMLtree]
optNesting list@(s:_) | not (isColor s) = list
optNesting list | null colorRanges = list
		| otherwise = optNesting (reorder smallest list)
  where colorIndex = accumArray f [] (Red,White) (zip (map getColor list) [0..])
  			where f = flip (:)
  	colorRanges = filter ((>1) . length . snd) (assocs colorIndex)
  	smallest = fst $ head $ sortBy rangeCmp colorRanges
  	rangeCmp (_,r1) (_,r2) = compare (head r1 - last r1) (head r2 - last r2)

getColor (TagNode (Color c) _) = c

reorder color list 
 = let (before,tmp) = span ((/= color) . getColor) list
       (revEnd,revMiddle) = span ((/= color) . getColor) (reverse tmp)
       --(middle,end) = splitAt ((+1) $ head $ reverse $ findIndices (==c) $ map getColor tmp) tmp
   in --before ++ [Tag (Color color) (concatMap flat middle)] ++ end
      before ++ [TagNode (Color color) (concatMap flat (reverse revMiddle))] ++ (reverse revEnd)
	where flat (TagNode (Color c) children) | c == color = children
	      flat tree = [tree]
