module Main(main) where

import AbsSML
import SMLtree
import Parser
import Printer
import IO
import IOExts
import Exception
import List (sortBy)

import WhiteRemover
import Eval
import ColorNest
import PLshortcut
import RedundancyElim
import SortSharing

main = do instring <- getContents
	  writeFile "/tmp/escape" instring
	  ws <- trypass whitepass
	  opt <- trypass optpass
	  let files = concat [ws,opt]
	  counter <- newIORef 0
	  improved <- sequence (map (trypass . improve counter) files)

	  handle <- chooseShortest ("/tmp/escape":files ++ concat improved)
	  out <- hGetContents handle
	  putStr out
	  hClose handle

trypass pass = Exception.catch pass (\_ -> return [])

whitepass = do h <- openFile "/tmp/escape" ReadMode
	       instring <- hGetContents h
	       let sml = parse instring
	       let noWhite = whiteRemover sml
	       wsfile <- openFile "/tmp/ws" WriteMode
	       Printer.print wsfile noWhite
	       hClose wsfile
	       return ["/tmp/ws"]

optpass = do h <- openFile "/tmp/escape" ReadMode
	     instring <- hGetContents h
	     let opt = optimize $ whiteMerge $ whiteRemover $ parse instring
	     optfile <- openFile "/tmp/opt" WriteMode
	     Printer.print optfile opt
	     hClose optfile
	     return ["/tmp/opt"]

improve c path
    = do file <- openFile path ReadMode
	 instring <- hGetContents file
	 let imptree = colorNest $ redundElim $ ast2tree $ whiteMerge $ parse instring
	 let multi = plshortcut $ sortSharing $ tree2multi imptree
	 let out = tree2ast (multi2tree multi)
	 newName <- readIORef c
	 modifyIORef c succ
	 let filepath = "/tmp/imp" ++ show newName
	 handle <- openFile filepath WriteMode
	 Printer.print handle out
	 hClose handle
	 return [filepath]

chooseShortest ls
    = do hs <- sequence $ map (\path -> openFile path ReadMode) ls
	 sizes <- sequence $ map hFileSize hs
	 let (short:rest) = sortBy (\a b -> compare (fst a) (fst b)) (zip sizes hs)
	 sequence_ $ map hClose (map snd rest)
	 return (snd short)
