module PLshortcut(plshortcut) where

import SMLtree
import State
import List

-- run preferrably after canonicalisation and redundancy elim

plshortcut :: [MultiTree] -> [MultiTree]
plshortcut = shortcut initState

plshort st (TagMulti tags ch) = TagMulti tags (map (plshort state) (shortcut state ch))
  where state = foldl newState st tags
plshort st s = s

shortcut state trees | not $ null candidates
  = reorder state trees $ fst3 $ head $
    sortBy (\(_,_,s1) (_,_,s2) -> compare s1 s2) candidates
  where candidates :: [([Int],[TagSort],Int)]
        candidates = filter (\(_,_,save) -> save > cost) $
		     map (\(index,tags) -> (index,tags,saving tags)) $
		     map (\(index,t:tags) -> (index,t)) $
		     map unzip $
		     groupBy (\a b -> (snd a) == (snd b)) $
		     sortBy (\a b -> compare (snd b) (snd a)) $
		     filter (all (`elem` interestingTags) . snd) $
		     filter (not . null . snd) $
		     zip [0..] (map getTags  trees)

	cost = plcost + trycost state

	getTags (TagMulti tags _) = tags
	getTags _ = []

	saving = sum . map ((*2) . SMLtree.cost)
shortcut state trees = trees

fst3 (a,_,_) = a

reorder state trees indices
  = let (before,tmp) = splitAt (head indices) trees
        (middle,after) = splitAt (Prelude.last indices - head indices + 1) tmp
        (tags,first,last,foo) = fix middle
    in before ++ 
       TagMulti tags (first ++ TagMulti (Plaintext:restore state) foo : last) :
       after

fix (TagMulti tags ch1:rest) = (tags,ch1,ch2,reverse revfoo)
  where (TagMulti _ ch2:revfoo) = reverse rest

restore state = add (b state) Bold $
		add (em state) Emphasis $
		add (i state) Italic $
		add (s state) Strong $
		add (tt state) TT $
		replicate (u state) Underline

plcost = 8

interestingTags = [Bold,Emphasis,Italic,Strong,TT,Underline]

trycost state =	sum $
                add (b state) 6 $
		add (em state) 8 $
		add (i state) 6 $
		add (s state) 6 $
		add (tt state) 8 $
		[u state * 6]

add cond elem = if cond then (elem:) else id

