module Parser(parse) where

import AbsSML
import Char
import PackedString

--This is really more like a lexer...

parse :: String -> [SMLNG]
parse ('\x20':ss) = WS SPC : parse ss
parse ('\x0D':ss) = WS CR : parse ss
parse ('\x0A':ss) = WS LF : parse ss
parse ('\x09':ss) = WS TAB : parse ss
parse ('<':'/':ss) = mkTag False ss
parse ('<':ss) = mkTag True ss
parse [] = []
parse ss = parseText ss
  where parseText :: String -> [SMLNG]
	parseText ss = let (text,ss') = span notSpaceNorTag ss
		       in Text (packString text) : parse ss'

        notSpaceNorTag c = not (isSpace c) && c /= '<'

mkTag :: Bool -> String -> [SMLNG]
mkTag b (s:ss) | isDigit s
    = Tag b (Size (fromEnum s - fromEnum '0')) : consumeGR ss
mkTag b ('B':ss) = Tag b Bold : consumeGR ss
mkTag b ('E':'M':ss) = Tag b Emphasis : consumeGR ss
mkTag b ('I':ss) = Tag b Italic : consumeGR ss
mkTag b ('P':'L':ss) = Tag b Plaintext : consumeGR ss
mkTag b ('S':ss) = Tag b Strong : consumeGR ss
mkTag b ('T':'T':ss) = Tag b TT : consumeGR ss
mkTag b ('U':ss) = Tag b Underline : consumeGR ss
mkTag b ('r':ss) = Tag b (Color Red) : consumeGR ss
mkTag b ('b':ss) = Tag b (Color Blue) : consumeGR ss
mkTag b ('g':ss) = Tag b (Color Green) : consumeGR ss
mkTag b ('c':ss) = Tag b (Color Cyan) : consumeGR ss
mkTag b ('m':ss) = Tag b (Color Mangenta) : consumeGR ss
mkTag b ('y':ss) = Tag b (Color Yellow) : consumeGR ss
mkTag b ('k':ss) = Tag b (Color Black) : consumeGR ss
mkTag b ('w':ss) = Tag b (Color White) : consumeGR ss
mkTag _ _ = error "Parse error!"

consumeGR ('>':ss) = parse ss
