module Printer(print) where

import AbsSML
import Prelude hiding (print)
import IO hiding (print)
import PackedString

print :: Handle -> [SMLNG] -> IO ()
print h (Text s:ss) = hPutStr h (unpackPS s) >> print h ss
print h (Tag b tag:ss) = putTag h b tag >> print h ss
print h (WS white:ss) = putWhite h white >> print h ss
print h [] = return ()

putTag h b tag = do hPutChar h '<'
		    if not b then hPutChar h '/'
		       else return ()
		    hPutStr h (tag2str tag)
		    hPutChar h '>'
  where tag2str Bold = "B"
	tag2str Emphasis = "EM"
	tag2str Italic = "I"
	tag2str Plaintext = "PL"
	tag2str Strong = "S"
	tag2str TT = "TT"
	tag2str Underline = "U"
	tag2str (Size i) = show i
	tag2str (Color c) = color2str c

        color2str Red = "r"
	color2str Green = "g"
	color2str Blue = "b"
	color2str Cyan = "c"
	color2str Mangenta = "m"
	color2str Yellow = "y"
	color2str Black = "b"
	color2str White = "w"
	color2str Dunno = error "Internal error! Strange color"

putWhite h SPC = hPutChar h '\x20'
putWhite h CR  = hPutChar h '\x0D'
putWhite h LF  = hPutChar h '\x0A'
putWhite h TAB = hPutChar h '\x09'

