module SMLtree(module AbsSML
              ,SMLtree(..)
              ,ast2tree
              ,tree2ast
              ,MultiTree(..)
              ,tree2multi
              ,multi2tree) where

import AbsSML
import PackedString

data SMLtree
	= TagNode TagSort [SMLtree]
	| TextLeaf PackedString
	| WSLeaf WhiteSpace
	  deriving (Eq,Show)

ast2tree :: [SMLNG] -> [SMLtree]
ast2tree = work []
  where work stack (Text s:ss) = work (Left (TextLeaf s):stack) ss
	work stack (WS ws:ss) = work (Left (WSLeaf ws):stack) ss
	work stack (Tag True tag:ss) = work (Right tag:stack) ss
	work stack (Tag False tag:ss) = work (reduce tag stack) ss
	work stack [] = reverse $ map stripLeft stack

        reduce tag stack 
	    = let (children,_:rest) = span (/=Right tag) stack
	      in Left (TagNode tag (reverse $ map stripLeft children)) : rest

stripLeft (Left a) = a

tree2ast :: [SMLtree] -> [SMLNG]
tree2ast ss = work ss []
  where work (TextLeaf s:ss) = (Text s :) . work ss
	work (WSLeaf ws:ss) = (WS ws:) . work ss
	work (TagNode tag children:ss)
	    = (Tag True tag:) . work children . (Tag False tag:) . work ss
	work [] = id

data MultiTree
	= TagMulti [TagSort] [MultiTree] -- invariant: tag list never empty
	| TextMulti PackedString
	| WSMulti WhiteSpace

tree2multi :: [SMLtree] -> [MultiTree]
tree2multi = map conv

conv (TextLeaf s) = TextMulti s
conv (WSLeaf ws) = WSMulti ws
conv (TagNode t ch) = TagMulti (t:tags) (tree2multi children)
  where (tags,children) = getTags ch

getTags [TagNode tag ch] = (tag:tags,rest)
  where (tags,rest) = getTags ch
getTags ls = ([],ls)

multi2tree :: [MultiTree] -> [SMLtree]
multi2tree = map revert

revert (TextMulti s) = TextLeaf s
revert (WSMulti ws) = WSLeaf ws
revert (TagMulti (t:ts) ch)
  = TagNode t (foldr (\tag tree -> [TagNode tag tree]) (multi2tree ch) ts)
