module WhiteRemover(whiteRemover, whiteMerge) where

import AbsSML
import PackedString

data SpaceContext
	= SC { u :: Int,
	       color :: Color,
	       tt :: Bool
	     } deriving Eq

whiteRemover :: [SMLNG] -> [SMLNG]
whiteRemover = wR [SC{u=0,color=Dunno,tt=False}] Nothing
  where wR state _ (Text s : ss) = Text s : wR state Nothing ss
  	wR (st:sts) prev (Tag False t: ss) = Tag False t: wR sts prev ss
  	wR state prev (Tag True t:ss) = Tag True t : wR (newState t state) prev ss
  	wR state@(st:_) Nothing (WS ws:ss) = WS SPC : wR state (Just st) ss
  	wR state@(st:_) prev@(Just ps) (WS ws:ss) = 
  		if st == ps && not (tt st) then
  			wR state prev ss
  		else
  			WS SPC : wR state (Just st) ss
  	wR _ _ [] = []
	wR [] _ _ = error "Internal error! Space context empty!"

newState :: TagSort -> [SpaceContext] -> [SpaceContext]
newState Plaintext (s:ss) = s{u=0,tt=False}:s:ss
newState TT (s:ss) = s{tt=True}:s:ss
newState Underline (s:ss) = s{u=incU (u s)}:s:ss
newState (Color c) (s:ss) = s{color=c}:s:ss
newState _ (s:ss) = s:s:ss
newState _ [] = error "Internal error! Space context empty!"

incU n | n < 3 = n+1
incU n         = n

-- A pass which puts whitespace in text to reduce the number of things the
-- other passes have to juggle with
-- Run preferrably after whiteRemover

whiteMerge :: [SMLNG] -> [SMLNG]
{- This one has quadratic behaviour!!!
whiteMerge (Text s: WS ws: Text s':ss)
	= whiteMerge (Text ((concatPS [s,packString " ",s']):ss)
whiteMerge (s:ss) = s : whiteMerge ss
whiteMerge [] = []
-}
whiteMerge stream@(s:_) | isText s = let (txt,rest) = span (not . isTag) stream
				     in Text (concatPS (map mkPS txt)) : whiteMerge rest
whiteMerge (s:ss) = s : whiteMerge ss
whiteMerge [] = []

mkPS (Text s) = s
mkPS (WS _) = whitePS
whitePS = packString " "

isText (Text _) = True
isText _ = False

isTag (Tag _ _) = True
isTag _ = False
